@@ -19,6 +19,7 @@
 
 package org.apache.cxf.jaxrs.utils;
 
+import java.io.Closeable;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -47,6 +48,7 @@ import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.stream.Collectors;
 
+import javax.activation.DataSource;
 import javax.ws.rs.ClientErrorException;
 import javax.ws.rs.Consumes;
 import javax.ws.rs.HttpMethod;
@@ -81,6 +83,7 @@ import javax.ws.rs.ext.ReaderInterceptorContext;
 import javax.ws.rs.ext.WriterInterceptor;
 import javax.ws.rs.ext.WriterInterceptorContext;
 import javax.xml.namespace.QName;
+import javax.xml.transform.Source;
 
 import org.apache.cxf.common.i18n.BundleUtils;
 import org.apache.cxf.common.logging.LogUtils;
@@ -169,12 +172,77 @@ public final class JAXRSUtils {
     private static final Annotation[] EMPTY_ANNOTATIONS = new Annotation[0];
     private static final Set<Class<?>> STREAMING_OUT_TYPES = new HashSet<>(
         Arrays.asList(InputStream.class, Reader.class, StreamingOutput.class));
+    private static final Set<String> STREAMING_LIKE_OUT_TYPES = new HashSet<>(
+        Arrays.asList(
+            "org.apache.cxf.jaxrs.ext.xml.XMLSource", 
+            "org.apache.cxf.jaxrs.ext.multipart.InputStreamDataSource", 
+            "org.apache.cxf.jaxrs.ext.multipart.MultipartBody", 
+            "org.apache.cxf.jaxrs.ext.multipart.Attachment"
+        ));
+    private static final Set<String> REACTIVE_OUT_TYPES = new HashSet<>(
+        Arrays.asList(
+            // Reactive Streams
+            "org.reactivestreams.Publisher",
+            // Project Reactor
+            "reactor.core.publisher.Mono",
+            "reactor.core.publisher.Flux",
+            // RxJava
+            "rx.Observable",
+            // RxJava2
+            "io.reactivex.Flowable",
+            "io.reactivex.Maybe",
+            "io.reactivex.Observable",
+            "io.reactivex.Single",
+            // RxJava3
+            "io.reactivex.rxjava3.core.Flowable",
+            "io.reactivex.rxjava3.core.Maybe",
+            "io.reactivex.rxjava3.core.Observable",
+            "io.reactivex.rxjava3.core.Single",
+            // JDK
+            "java.util.concurrent.CompletableFuture",
+            "java.util.concurrent.CompletionStage"
+        ));
 
     private JAXRSUtils() {
     }
 
+    /**
+     * Consider additional types as stream-like and returns if the class and corresponding type
+     * refer to one of those. 
+     * @param cls class to check
+     * @param type type to check
+     * @return "true" is the class and corresponding type could be considered streaming-like, 
+     * "false" otherwise.
+     */
+    public static boolean isStreamingLikeOutType(Class<?> cls, Type type) {
+        if (cls != null && (isStreamingOutType(cls) 
+                || STREAMING_LIKE_OUT_TYPES.contains(cls.getName()) 
+                || REACTIVE_OUT_TYPES.contains(cls.getName()))) {
+            return true;
+        }
+
+        if (type instanceof ParameterizedType) {
+            final ParameterizedType parameterizedType = (ParameterizedType)type;
+            for (Type arg: parameterizedType.getActualTypeArguments()) {
+                if (isStreamingLikeOutType(null, arg)) {
+                    return true;
+                }
+            }
+        }
+        
+        if (type instanceof Class) {
+            final Class<?> typeCls = (Class<?>)type;
+            return isStreamingOutType(typeCls) || STREAMING_LIKE_OUT_TYPES.contains(typeCls.getName());
+        }
+        
+        return false;
+    }
+    
     public static boolean isStreamingOutType(Class<?> type) {
-        return STREAMING_OUT_TYPES.contains(type);
+        return STREAMING_OUT_TYPES.contains(type) 
+            || Closeable.class.isAssignableFrom(type)
+            || Source.class.isAssignableFrom(type)
+            || DataSource.class.isAssignableFrom(type);
     }
 
     public static List<PathSegment> getPathSegments(String thePath, boolean decode) {
@@ -442,7 +510,7 @@ public final class JAXRSUtils {
         int pathMatched = 0;
         int methodMatched = 0;
         int consumeMatched = 0;
-
+        
         List<OperationResourceInfo> finalPathSubresources = null;
         for (Map.Entry<ClassResourceInfo, MultivaluedMap<String, String>> rEntry : matchedResources.entrySet()) {
             ClassResourceInfo resource = rEntry.getKey();
@@ -493,7 +561,12 @@ public final class JAXRSUtils {
                 LOG.fine(matchMessageLogSupplier(ori, path, httpMethod, requestType, acceptContentTypes, added));
             }
         }
-        if (finalPathSubresources != null && pathMatched > 0
+        
+        // We may get several matching candidates with different HTTP methods which match subresources
+        // and resources. Before excluding subresources, let us make sure we have at least one matching
+        // HTTP method candidate.
+        boolean isOptions = HttpMethod.OPTIONS.equalsIgnoreCase(httpMethod);
+        if (finalPathSubresources != null && (methodMatched > 0 || isOptions)
             && !MessageUtils.getContextualBoolean(message, KEEP_SUBRESOURCE_CANDIDATES, false)) {
             for (OperationResourceInfo key : finalPathSubresources) {
                 candidateList.remove(key);
@@ -1078,15 +1151,16 @@ public final class JAXRSUtils {
         MultivaluedMap<String, String> params =
             (MultivaluedMap<String, String>)m.get(FormUtils.FORM_PARAM_MAP);
 
+        String enc = HttpUtils.getEncoding(mt, StandardCharsets.UTF_8.name());
         if (params == null) {
             params = new MetadataMap<>();
             m.put(FormUtils.FORM_PARAM_MAP, params);
 
             if (mt == null || mt.isCompatible(MediaType.APPLICATION_FORM_URLENCODED_TYPE)) {
                 InputStream entityStream = copyAndGetEntityStream(m);
-                String enc = HttpUtils.getEncoding(mt, StandardCharsets.UTF_8.name());
                 String body = FormUtils.readBody(entityStream, enc);
-                FormUtils.populateMapFromStringOrHttpRequest(params, m, body, enc, decode);
+                // Do not decode unless the key is empty value, fe @FormParam("")
+                FormUtils.populateMapFromStringOrHttpRequest(params, m, body, enc, StringUtils.isEmpty(key) && decode);
             } else {
                 if ("multipart".equalsIgnoreCase(mt.getType())
                     && MediaType.MULTIPART_FORM_DATA_TYPE.isCompatible(mt)) {
@@ -1103,6 +1177,14 @@ public final class JAXRSUtils {
             }
         }
 
+        if (decode && !MessageUtils.getContextualBoolean(m, FormUtils.FORM_PARAM_MAP_DECODED, false)) {
+            List<String> values = params.get(key);
+            if (values != null) {
+                values = values.stream().map(value -> HttpUtils.urlDecode(value, enc)).collect(Collectors.toList());
+                params.replace(key, values);
+            }
+        }
+
         if ("".equals(key)) {
             return InjectionUtils.handleBean(pClass, paramAnns, params, ParameterType.FORM, m, false);
         }
@@ -1342,7 +1424,6 @@ public final class JAXRSUtils {
 
     /**
      * Retrieve map of query parameters from the passed in message
-     * @param message
      * @return a Map of query parameters.
      */
     public static MultivaluedMap<String, String> getStructuredParams(String query,
@@ -2045,7 +2126,7 @@ public final class JAXRSUtils {
             return parent;
         } else if (parent.endsWith("/")) {
             // Remove only last slash
-            return parent.replaceAll("/$", "") + child;
+            return parent.substring(0, parent.length() - 1) + child;
         } else {
             return parent + child;
         }
